@@ -2,22 +2,39 @@ import re
 import warnings
 from django.conf import settings
 from django.utils.importlib import import_module
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from tenant_schemas.utils import get_public_schema_name, get_limit_set_calls
+from tenant_schemas.postgresql_backend.introspection import DatabaseSchemaIntrospection
+import django.db.utils
+import psycopg2
 
 ORIGINAL_BACKEND = getattr(settings, 'ORIGINAL_BACKEND', 'django.db.backends.postgresql_psycopg2')
 
-original_backend = import_module('.base', ORIGINAL_BACKEND)
+original_backend = import_module(ORIGINAL_BACKEND + '.base')
 
 EXTRA_SEARCH_PATHS = getattr(settings, 'PG_EXTRA_SEARCH_PATHS', [])
 
 # from the postgresql doc
-SQL_IDENTIFIER_RE = re.compile('^[_a-zA-Z][_a-zA-Z0-9]{,62}$')
+SQL_IDENTIFIER_RE = re.compile(r'^[_a-zA-Z][_a-zA-Z0-9]{,62}$')
+SQL_SCHEMA_NAME_RESERVED_RE = re.compile(r'^pg_', re.IGNORECASE)
+
+
+def _is_valid_identifier(identifier):
+    return bool(SQL_IDENTIFIER_RE.match(identifier))
 
 
 def _check_identifier(identifier):
-    if not SQL_IDENTIFIER_RE.match(identifier):
-        raise RuntimeError("Invalid string used for the schema name.")
+    if not _is_valid_identifier(identifier):
+        raise ValidationError("Invalid string used for the identifier.")
+
+
+def _is_valid_schema_name(name):
+    return _is_valid_identifier(name) and not SQL_SCHEMA_NAME_RESERVED_RE.match(name)
+
+
+def _check_schema_name(name):
+    if not _is_valid_schema_name(name):
+        raise ValidationError("Invalid string used for the schema name.")
 
 
 class DatabaseWrapper(original_backend.DatabaseWrapper):
@@ -28,6 +45,10 @@ class DatabaseWrapper(original_backend.DatabaseWrapper):
 
     def __init__(self, *args, **kwargs):
         super(DatabaseWrapper, self).__init__(*args, **kwargs)
+
+        # Use a patched version of the DatabaseIntrospection that only returns the table list for the
+        # currently selected schema.
+        self.introspection = DatabaseSchemaIntrospection(self)
         self.set_schema_to_public()
 
     def set_tenant(self, tenant, include_public=True):
@@ -60,7 +81,7 @@ class DatabaseWrapper(original_backend.DatabaseWrapper):
         self.schema_name = get_public_schema_name()
         self.set_settings_schema(self.schema_name)
         self.search_path_set = False
-        
+
     def set_settings_schema(self, schema_name):
         self.settings_dict['SCHEMA'] = schema_name
 
@@ -90,7 +111,7 @@ class DatabaseWrapper(original_backend.DatabaseWrapper):
             if not self.schema_name:
                 raise ImproperlyConfigured("Database schema not set. Did you forget "
                                            "to call set_schema() or set_tenant()?")
-            _check_identifier(self.schema_name)
+            _check_schema_name(self.schema_name)
             public_schema_name = get_public_schema_name()
             search_paths = []
 
@@ -102,8 +123,16 @@ class DatabaseWrapper(original_backend.DatabaseWrapper):
                 search_paths = [self.schema_name]
 
             search_paths.extend(EXTRA_SEARCH_PATHS)
-            cursor.execute('SET search_path = {0}'.format(','.join(search_paths)))
-            self.search_path_set = True
+            # In the event that an error already happened in this transaction and we are going
+            # to rollback we should just ignore database error when setting the search_path
+            # if the next instruction is not a rollback it will just fail also, so
+            # we do not have to worry that it's not the good one
+            try:
+                cursor.execute('SET search_path = {0}'.format(','.join(search_paths)))
+            except (django.db.utils.DatabaseError, psycopg2.InternalError):
+                self.search_path_set = False
+            else:
+                self.search_path_set = True
         return cursor
 
 
